from typing import Optional

from lazynote.manager.base import BaseManager


class CustomManager(BaseManager):
    def gen_docstring(self, old_docstring: Optional[str], pattern: str, node_code: str) -> str:
        """
        自定义生成新的文档字符串的逻辑。

        :param old_docstring: 旧的文档字符串
        :param pattern: 模式字符串
        :param node_code: 节点代码
        :return: 新的文档字符串
        """
        if old_docstring:
            return f"{old_docstring}\n\n{pattern}"
        else:
            return f"{pattern}"



import importlib
import sys
from pathlib import Path

import click
from click import Context

from lazynote import __version__
from lazynote.config import settings
from lazynote.log import init_log
from lazynote.manager import SimpleManager


def import_module_or_package(name_or_path):
    path = Path(name_or_path)
    if path.is_dir():
        # Assume it's a package path
        package_name = path.name
        package_dir = str(path.resolve().parent)
        sys.path.append(package_dir)
        module_or_package = importlib.import_module(package_name)
    else:
        # Assume it's a module name
        module_or_package = importlib.import_module(name_or_path)
    return module_or_package

def run_function(module_or_package, pattern, skip_modules):
    manager = SimpleManager(pattern=pattern)
    manager.traverse(module_or_package, skip_modules=skip_modules)

@click.group(invoke_without_command=True)
@click.pass_context
@click.option(
    '-V',
    '--version',
    is_flag=True,
    help='Show version and exit.'
)  # If it's true, it will override `settings.VERBOSE`
@click.option('-v', '--verbose', is_flag=True, help='Show more info.')
@click.option(
    '--debug',
    is_flag=True,
    help='Enable debug.'
)  # If it's true, it will override `settings.DEBUG`
def main(ctx: Context, version: str, verbose: bool, debug: bool):
    """Main commands"""
    if version:
        click.echo(__version__)
    elif ctx.invoked_subcommand is None:
        click.echo(ctx.get_help())
        # Manually invoke the help of the run command
        click.echo('\nRun command help:')
        click.echo(ctx.command.get_command(ctx, 'run').get_help(ctx))
    else:
        if verbose:
            settings.set('VERBOSE', True)
        if debug:
            settings.set('DEBUG', True)

@main.command()
@click.argument('name_or_path')
@click.option('--pattern', default='fill', help='Pattern for SimpleManager')
@click.option('--skip-modules', default='', help='Comma-separated list of modules to skip')
def run(name_or_path, pattern, skip_modules):
    """Run command"""
    init_log()
    click.echo(f'Running with {name_or_path}')
    skip_modules_list = skip_modules.split(',') if skip_modules else []
    try:
        module_or_package = import_module_or_package(name_or_path)
        run_function(module_or_package, pattern, skip_modules_list)
        click.echo('Run completed successfully.')
    except Exception as e:
        click.echo(f'Error: {e}')


if __name__ == "__main__":
    main()



"""
Configuration center.
Use https://www.dynaconf.com/
"""""
import os
import sys
from pathlib import Path

from dynaconf import Dynaconf


_base_dir = Path(__file__).parent.parent

_settings_files = [
    # All config file will merge.
    Path(__file__).parent / 'settings.yml',  # Load default config.
]

# User configuration. It will be created automatically by the pip installer .
_external_files = [
    Path(sys.prefix, 'etc', 'lazynote', 'settings.yml')
]


settings = Dynaconf(
    # Set env `LAZYNOTE_FOO='bar'`，use `settings.FOO` .
    envvar_prefix='LAZYNOTE',
    settings_files=_settings_files,  # load user configuration.
    # environments=True,  # Enable multi-level configuration，eg: default, development, production
    load_dotenv=True,  # Enable load .env
    # env_switcher='LAZYNOTE_ENV',
    lowercase_read=False,  # If true, can't use `settings.foo`, but can only use `settings.FOO`
    includes=_external_files,  # Customs settings.
    base_dir=_base_dir,  # `settings.BASE_DIR`
)



import inspect
from typing import Callable, Optional

import libcst as cst
import libcst.matchers as m


class BaseEditor(cst.CSTTransformer):
    '''
    A tool for code text Transformer new code text
    '''

    def __init__(self, gen_docstring: Callable, pattern: str, module):
        self.gen_docstring = gen_docstring
        self.pattern = pattern
        self.module = module
        self.module_dict = self.create_module_dict(module)
        self.current_class = None

    def create_module_dict(self, module):
        module_dict = {}
        seen_objects = set()
        for name, obj in inspect.getmembers(module):
            module_dict[name] = obj
            if inspect.isclass(obj):
                self.add_class_members_to_dict(
                    module_dict, obj, name, seen_objects)
        return module_dict

    def add_class_members_to_dict(self, module_dict, cls, parent_name, seen_objects):
        if cls in seen_objects:
            return
        seen_objects.add(cls)
        for name, obj in inspect.getmembers(cls):
            full_name = f"{parent_name}.{name}"
            module_dict[full_name] = obj
            if inspect.isclass(obj):
                self.add_class_members_to_dict(
                    module_dict, obj, full_name, seen_objects)

    def leave_FunctionDef(self, original_node: cst.FunctionDef, updated_node: cst.FunctionDef) -> cst.FunctionDef:
        full_name = f"{self.current_class}.{original_node.name.value}" if self.current_class else original_node.name.value
        obj = self._get_obj_by_name(full_name)
        docstring = obj.__doc__ if obj else None
        return self._update_node_with_new_docstring(original_node, updated_node, docstring)

    def visit_ClassDef(self, node: cst.ClassDef) -> None:
        self.current_class = node.name.value

    def leave_ClassDef(self, original_node: cst.ClassDef, updated_node: cst.ClassDef) -> cst.ClassDef:
        self.current_class = None
        obj = self._get_obj_by_name(original_node.name.value)
        docstring = obj.__doc__ if obj else None
        return self._update_node_with_new_docstring(original_node, updated_node, docstring)

    def leave_Module(self, original_node: cst.Module, updated_node: cst.Module) -> cst.Module:
        return self._update_node_with_new_docstring(original_node, updated_node, self.module.__doc__)

    def _get_obj_by_name(self, name: str):
        return self.module_dict.get(name, None)

    def _update_node_with_new_docstring(self, original_node: cst.CSTNode, updated_node: cst.CSTNode, docstring: Optional[str]) -> cst.CSTNode:

        node_code = cst.Module([]).code_for_node(original_node)

        old_docstring = docstring
        new_body = []

        if isinstance(updated_node.body, tuple):
            body = updated_node.body
        else:
            body = getattr(updated_node.body, 'body', [])

        # Extract existing docstring if present and build new body without it
        for stmt in body:
            if m.matches(stmt, m.SimpleStatementLine(body=[m.Expr(m.SimpleString())])):
                old_docstring = cst.ensure_type(
                    stmt.body[0].value, cst.SimpleString).value.strip('\"\'')
            else:
                new_body.append(stmt)

        new_docstring = self.gen_docstring(
            old_docstring, node_code)

        # Create a new docstring node if new_docstring is provided
        new_docstring_node = (
            cst.SimpleStatementLine([cst.Expr(cst.SimpleString(
                f'"""{new_docstring}"""'))]) if new_docstring else None
        )

        # Insert the new docstring at the beginning of the body if it exists
        if new_docstring_node:
            new_body.insert(0, new_docstring_node)

        # Update the body with the new list of statements
        if isinstance(updated_node.body, tuple):
            updated_body = tuple(new_body)
        else:
            updated_body = updated_node.body.with_changes(body=new_body)

        return updated_node.with_changes(body=updated_body)



import importlib
import pkgutil
from typing import Callable, Dict

from lazynote.schema import MemberType, get_member_type


class BaseParser:
    def __init__(self, skip_modules):
        self.parsers: Dict[MemberType, Callable] = {
            MemberType.PACKAGE: self.parse_package,
            MemberType.MODULE: self.parse_module,
            MemberType.CLASS: self.parse_class,
            MemberType.METHOD: self.parse_method,
            MemberType.FUNCTION: self.parse_function,
            MemberType.PROPERTY: self.parse_property,
            MemberType.ATTRIBUTE: self.parse_attribute
        }

    def parse(self, member, module, manager, **kwargs):
        member_type = get_member_type(member)
        parser = self.parsers.get(member_type)
        if parser:
            parser(member, module, manager, **kwargs)

    def parse_package(self, package, module, manager, **kwargs):
        skip_modules = kwargs.get('skip_modules', [])
        print(f"Package: {package.__name__}")
        for importer, modname, ispkg in pkgutil.walk_packages(package.__path__, package.__name__ + "."):
            if modname in skip_modules:
                print(f"Skipping module: {modname}")
                continue
            try:
                submodule = importlib.import_module(modname)
                manager.traverse(submodule, skip_modules)
            except Exception as e:
                print(f"Error importing {modname}: {e}")

    def parse_module(self, module, parent_module, manager,**kwargs):
        print(f"--Module: {module.__name__}--")
        new_code = manager.modify_docstring(module)
        print(new_code[:20])

    def parse_class(self, cls, module, manager,**kwargs):
        print(f"Class: {cls.__name__}")

    def parse_method(self, method, module, manager,**kwargs):
        print(f"  Method: {method.__name__}")

    def parse_function(self, func, module, manager,**kwargs):
        print(f"Function: {func.__name__}")

    def parse_property(self, prop, module, manager,**kwargs):
        print(f"  Property: {prop}")

    def parse_attribute(self, attr, module, manager,**kwargs):
        print(f"  Attribute: {attr}")



import inspect
import textwrap
from abc import ABC, abstractmethod
from typing import Optional

import libcst as cst
from pydantic import BaseModel, Field

from lazynote.parser import BaseParser
from lazynote.schema import MemberType, get_member_type


class BaseManager(BaseModel, ABC):
    """
    执行器，用于修改模块的文档字符串，目前只支持模块级别或文件级别。

    子类需要重写 gen_docstring 方法以生成自定义的文档字符串。
    """

    parser: Optional[BaseParser] = Field(default=None)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if self.parser is None:
            self.parser = BaseParser(skip_modules=kwargs.get('skip_modules', []))

    class Config:
        arbitrary_types_allowed = True

    @abstractmethod
    def gen_docstring(self, old_docstring: Optional[str], node_code: str) -> str:
        """
        生成新的文档字符串。子类必须实现此方法以实现自定义逻辑。

        :param old_docstring: 旧的文档字符串
        :param node_code: 节点代码
        :return: 新的文档字符串
        """
        pass

    @staticmethod
    def is_defined_in_module(member, module):
        if hasattr(member, '__module__'):
            return member.__module__ == module.__name__
        elif isinstance(member, property):
            return member.fget.__module__ == module.__name__
        elif isinstance(member, staticmethod):
            return member.__func__.__module__ == module.__name__
        elif isinstance(member, classmethod):
            return member.__func__.__module__ == module.__name__
        elif hasattr(member, '__wrapped__'):
            return member.__wrapped__.__module__ == module.__name__
        return False
    
    @abstractmethod
    def modify_docstring(self, module):
        pass

    def _write_code_to_file(self, module, code: str):
        # 获取模块文件的路径
        module_file_path = inspect.getfile(module)

        # 将修改后的代码写回文件
        with open(module_file_path, 'w', encoding='utf-8') as file:
            file.write(code)

    def traverse(self, obj, skip_modules=None):
        if skip_modules is None:
            skip_modules = []

        member_type = get_member_type(obj)
        module = inspect.getmodule(obj)

        if module and module.__name__ in skip_modules:
            print(f"Skipping module: {module.__name__}")
            return

        # 直接使用 BaseParser 作为工具类
        self.parser.parse(obj, module, self,skip_modules=skip_modules)

        if member_type in (MemberType.PACKAGE):
            for name, member in inspect.getmembers(obj):
                if self.is_defined_in_module(member, module):
                    self.traverse(member, skip_modules=skip_modules)



import inspect
import textwrap
from enum import Enum
from typing import Callable, Optional

import libcst as cst

from lazynote.manager.base import BaseManager


class DocstringMode(str, Enum):
    TRANSLATE = "translate"
    POLISH = "polish"
    CLEAR = "clear"
    FILL = "fill"

class DocstringHandler:
    @staticmethod
    def handle_translate(old_docstring: Optional[str], node_code: str) -> str:
        # TODO
        return f"Translated: {old_docstring}" or None

    @staticmethod
    def handle_polish(old_docstring: Optional[str], node_code: str) -> str:
        # 实现润色逻辑
        # TODO
        return f"Polished: {old_docstring }" or None

    @staticmethod
    def handle_clear(old_docstring: Optional[str], node_code: str) -> str:
        return None

    @staticmethod
    def handle_fill(old_docstring: Optional[str], node_code: str) -> str:
        # 实现填入逻辑
        if old_docstring:
            return f"{old_docstring}"
        else:
            return None

    @staticmethod
    def get_handler(pattern: DocstringMode) -> Callable[[Optional[str], str], str]:
        try:
            # 自动映射模式到相应的处理函数
            handler_method_name = f"handle_{pattern.value}"
            return getattr(DocstringHandler, handler_method_name)
        except AttributeError:
            raise ValueError(f"No handler found for pattern: {pattern}")

class SimpleManager(BaseManager):
    """
        Initialize the SimpleManager.

        :param pattern: The docstring handling pattern. Should be one of:
                        - DocstringMode.TRANSLATE
                        - DocstringMode.POLISH
                        - DocstringMode.CLEAR
                        - DocstringMode.FILL
    """
    pattern: DocstringMode

    def gen_docstring(self, old_docstring: Optional[str], node_code: str) -> str:

        handler = DocstringHandler.get_handler(self.pattern)

        return handler(old_docstring, node_code)

    def modify_docstring(self, module):

        source_code = inspect.getsource(module)
        source_code = textwrap.dedent(source_code)  # 去除多余的缩进
        tree = cst.parse_module(source_code)

        from lazynote.editor import \
            BaseEditor  # Lazy import to avoid circular dependency
        transformer = BaseEditor(
            gen_docstring=self.gen_docstring, pattern=self.pattern,module=module)
        modified_tree = tree.visit(transformer)
        self._write_code_to_file(module, modified_tree.code)
        return modified_tree.code



"""Log"""
import logging
import os
from logging.config import dictConfig

from lazynote.config import settings

os.makedirs(settings.LOGPATH, exist_ok=True)


def verbose_formatter(verbose: int) -> str:
    """formatter factory"""
    if verbose is True:
        return 'verbose'
    return 'simple'


def update_log_level(debug: bool, level: str) -> str:
    """update log level"""
    if debug is True:
        level_num = logging.DEBUG
    else:
        level_num = logging.getLevelName(level)
    settings.set('LOGLEVEL', logging.getLevelName(level_num))
    return settings.LOGLEVEL


def init_log() -> None:
    """Init log config."""
    log_level = update_log_level(settings.DEBUG, str(settings.LOGLEVEL).upper())

    log_config = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            'verbose': {
                'format': '%(asctime)s %(levelname)s %(name)s %(process)d %(thread)d %(message)s',
            },
            'simple': {
                'format': '%(asctime)s %(levelname)s %(name)s %(message)s',
            },
        },
        "handlers": {
            "console": {
                "formatter": verbose_formatter(settings.VERBOSE),
                'level': 'DEBUG',
                "class": "logging.StreamHandler",
            },
            'file': {
                'class': 'logging.handlers.RotatingFileHandler',
                'level': 'DEBUG',
                'formatter': verbose_formatter(settings.VERBOSE),
                'filename': os.path.join(settings.LOGPATH, 'all.log'),
                'maxBytes': 1024 * 1024 * 1024 * 200,  # 200M
                'backupCount': '5',
                'encoding': 'utf-8'
            },
        },
        "loggers": {
            '': {'level': log_level, 'handlers': ['console']},
        }
    }

    dictConfig(log_config)



import inspect
from enum import Enum


class MemberType(str, Enum):
    PACKAGE = "package"
    MODULE = "module"
    CLASS = "class"
    METHOD = "method"
    FUNCTION = "function"
    ATTRIBUTE = "attribute"
    PROPERTY = "property"


def get_member_type(member):
    member_checks = {
        MemberType.PACKAGE: lambda m: inspect.ismodule(m) and hasattr(m, '__path__'),
        MemberType.MODULE: inspect.ismodule,
        MemberType.CLASS: inspect.isclass,
        MemberType.METHOD: inspect.ismethod,
        MemberType.FUNCTION: inspect.isfunction,
        MemberType.PROPERTY: lambda m: isinstance(m, property)
    }

    for member_type, check in member_checks.items():
        if check(member):
            return member_type
    return MemberType.ATTRIBUTE
